package javax.swing.text.html;

import java.util.HashMap;
import javax.swing.event.ChangeListener;
import javax.swing.text.Document;
import java.awt.FontMetrics;
import sun.swing.SwingUtilities2;
import java.awt.Component;
import java.awt.Shape;
import javax.swing.text.StyledDocument;
import javax.swing.UIManager;
import java.net.MalformedURLException;
import java.util.StringTokenizer;
import javax.swing.Icon;
import java.awt.Rectangle;
import java.awt.Container;
import javax.swing.border.BevelBorder;
import java.awt.Graphics;
import java.awt.Insets;
import java.io.Serializable;
import java.util.EmptyStackException;
import java.util.Stack;
import javax.swing.border.EmptyBorder;
import javax.swing.ImageIcon;
import java.awt.Color;
import java.awt.Font;
import javax.swing.text.MutableAttributeSet;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import javax.swing.plaf.UIResource;
import java.util.Enumeration;
import javax.swing.text.View;
import java.io.Reader;
import javax.swing.text.SimpleAttributeSet;
import java.io.IOException;
import java.io.StringReader;
import javax.swing.text.AttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.Style;
import javax.swing.text.Element;
import java.net.URL;
import java.util.Vector;
import java.util.Hashtable;
import javax.swing.border.Border;
import javax.swing.text.StyleContext;

public class StyleSheet extends StyleContext
{
    static final Border noBorder;
    static final int DEFAULT_FONT_SIZE = 3;
    private CSS css;
    private StyleSheet.SelectorMapping selectorMapping;
    private Hashtable<String, StyleSheet.ResolvedStyle> resolvedStyles;
    private Vector<StyleSheet> linkedStyleSheets;
    private URL base;
    static final int[] sizeMapDefault;
    private int[] sizeMap;
    private boolean w3cLengthUnits;
    
    public StyleSheet() {
        this.sizeMap = StyleSheet.sizeMapDefault;
        this.w3cLengthUnits = false;
        this.selectorMapping = new StyleSheet.SelectorMapping(0);
        this.resolvedStyles = new Hashtable();
        if (this.css == null) {
            this.css = new CSS();
        }
    }
    
    public Style getRule(HTML.Tag tag, Element parentElement) {
        StyleSheet.SearchBuffer obtainSearchBuffer = StyleSheet.SearchBuffer.obtainSearchBuffer();
        try {
            Vector<Element> vector = obtainSearchBuffer.getVector();
            for (; parentElement != null; parentElement = parentElement.getParentElement()) {
                vector.addElement(parentElement);
            }
            int size = vector.size();
            StringBuffer stringBuffer = obtainSearchBuffer.getStringBuffer();
            for (int i = size - 1; i >= 1; --i) {
                parentElement = (Element)vector.elementAt(i);
                AttributeSet attributes = parentElement.getAttributes();
                stringBuffer.append(attributes.getAttribute(StyleConstants.NameAttribute).toString());
                if (attributes != null) {
                    if (attributes.isDefined(HTML.Attribute.ID)) {
                        stringBuffer.append('#');
                        stringBuffer.append(attributes.getAttribute(HTML.Attribute.ID));
                    }
                    else if (attributes.isDefined(HTML.Attribute.CLASS)) {
                        stringBuffer.append('.');
                        stringBuffer.append(attributes.getAttribute(HTML.Attribute.CLASS));
                    }
                }
                stringBuffer.append(' ');
            }
            stringBuffer.append(tag.toString());
            parentElement = (Element)vector.elementAt(0);
            AttributeSet attributes2 = parentElement.getAttributes();
            if (parentElement.isLeaf()) {
                Object attribute = attributes2.getAttribute(tag);
                if ((attribute instanceof AttributeSet)) {
                    attributes2 = (AttributeSet)attribute;
                }
                else {
                    attributes2 = null;
                }
            }
            if (attributes2 != null) {
                if (attributes2.isDefined(HTML.Attribute.ID)) {
                    stringBuffer.append('#');
                    stringBuffer.append(attributes2.getAttribute(HTML.Attribute.ID));
                }
                else if (attributes2.isDefined(HTML.Attribute.CLASS)) {
                    stringBuffer.append('.');
                    stringBuffer.append(attributes2.getAttribute(HTML.Attribute.CLASS));
                }
            }
            return resolvedStyle = this.getResolvedStyle(stringBuffer.toString(), vector, tag);
        }
        finally {
            StyleSheet.SearchBuffer.releaseSearchBuffer(obtainSearchBuffer);
        }
    }
    
    public Style getRule(String cleanSelectorString) {
        cleanSelectorString = this.cleanSelectorString(cleanSelectorString);
        if (cleanSelectorString != null) {
            return this.getResolvedStyle(cleanSelectorString);
        }
        return null;
    }
    
    public void addRule(String s) {
        if (s != null) {
            if (s == "BASE_SIZE_DISABLE") {
                this.sizeMap = StyleSheet.sizeMapDefault;
            }
            else if (s.startsWith("BASE_SIZE ")) {
                this.rebaseSizeMap(Integer.parseInt(s.substring("BASE_SIZE ".length())));
            }
            else if (s == "W3C_LENGTH_UNITS_ENABLE") {
                this.w3cLengthUnits = true;
            }
            else if (s == "W3C_LENGTH_UNITS_DISABLE") {
                this.w3cLengthUnits = false;
            }
            else {
                StyleSheet.CssParser cssParser = new StyleSheet.CssParser();
                try {
                    cssParser.parse(this.getBase(), new StringReader(s), false, false);
                }
                catch (IOException ex) {}
            }
        }
    }
    
    public AttributeSet getDeclaration(String s) {
        if (s == null) {
            return SimpleAttributeSet.EMPTY;
        }
        return new StyleSheet.CssParser().parseDeclaration(s);
    }
    
    public void loadRules(Reader reader, URL url) throws IOException {
        new StyleSheet.CssParser().parse(url, reader, false, false);
    }
    
    public AttributeSet getViewAttributes(View view) {
        return new StyleSheet.ViewAttributeSet(view);
    }
    
    @Override
    public void removeStyle(String s) {
        if (this.getStyle(s) != null) {
            String[] simpleSelectors = this.getSimpleSelectors(this.cleanSelectorString(s));
            synchronized (this) {
                StyleSheet.SelectorMapping selectorMapping = this.getRootSelectorMapping();
                for (int i = simpleSelectors.length - 1; i >= 0; --i) {
                    selectorMapping = selectorMapping.getChildSelectorMapping(simpleSelectors[i], true);
                }
                Style style = selectorMapping.getStyle();
                if (style != null) {
                    selectorMapping.setStyle(null);
                    if (this.resolvedStyles.size() > 0) {
                        Enumeration<StyleSheet.ResolvedStyle> elements = this.resolvedStyles.elements();
                        while (elements.hasMoreElements()) {
                            ((StyleSheet.ResolvedStyle)elements.nextElement()).removeStyle(style);
                        }
                    }
                }
            }
        }
        super.removeStyle(s);
    }
    
    public void addStyleSheet(StyleSheet styleSheet) {
        synchronized (this) {
            if (this.linkedStyleSheets == null) {
                this.linkedStyleSheets = new Vector();
            }
            if (!this.linkedStyleSheets.contains(styleSheet)) {
                int index = 0;
                if (styleSheet instanceof UIResource && this.linkedStyleSheets.size() > 1) {
                    index = this.linkedStyleSheets.size() - 1;
                }
                this.linkedStyleSheets.insertElementAt(styleSheet, index);
                this.linkStyleSheetAt(styleSheet, index);
            }
        }
    }
    
    public void removeStyleSheet(StyleSheet o) {
        synchronized (this) {
            if (this.linkedStyleSheets != null) {
                int index = this.linkedStyleSheets.indexOf(o);
                if (index != -1) {
                    this.linkedStyleSheets.removeElementAt(index);
                    this.unlinkStyleSheet(o, index);
                    if (index == 0 && this.linkedStyleSheets.size() == 0) {
                        this.linkedStyleSheets = null;
                    }
                }
            }
        }
    }
    
    public StyleSheet[] getStyleSheets() {
        StyleSheet[] anArray;
        synchronized (this) {
            if (this.linkedStyleSheets != null) {
                anArray = new StyleSheet[this.linkedStyleSheets.size()];
                this.linkedStyleSheets.copyInto(anArray);
            }
            else {
                anArray = null;
            }
        }
        return anArray;
    }
    
    public void importStyleSheet(URL url) {
        try {
            InputStream openStream = url.openStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(openStream));
            new StyleSheet.CssParser().parse(url, bufferedReader, false, true);
            bufferedReader.close();
            openStream.close();
        }
        catch (Throwable t) {}
    }
    
    public void setBase(URL base) {
        this.base = base;
    }
    
    public URL getBase() {
        return this.base;
    }
    
    public void addCSSAttribute(MutableAttributeSet set, CSS.Attribute attribute, String s) {
        this.css.addInternalCSSValue(set, attribute, s);
    }
    
    public boolean addCSSAttributeFromHTML(MutableAttributeSet set, CSS.Attribute attribute, String s) {
        Object cssValue = this.css.getCssValue(attribute, s);
        if (cssValue != null) {
            set.addAttribute(attribute, cssValue);
            return true;
        }
        return false;
    }
    
    public AttributeSet translateHTMLToCSS(AttributeSet set) {
        AttributeSet translateHTMLToCSS = this.css.translateHTMLToCSS(set);
        Style addStyle = this.addStyle(null, null);
        addStyle.addAttributes(translateHTMLToCSS);
        return addStyle;
    }
    
    @Override
    public AttributeSet addAttribute(AttributeSet removeAttribute, Object name, Object value) {
        if (this.css == null) {
            this.css = new CSS();
        }
        if ((name instanceof StyleConstants)) {
            HTML.Tag tagForStyleConstantsKey = HTML.getTagForStyleConstantsKey((StyleConstants)name);
            if (tagForStyleConstantsKey != null && removeAttribute.isDefined(tagForStyleConstantsKey)) {
                removeAttribute = this.removeAttribute(removeAttribute, tagForStyleConstantsKey);
            }
            Object styleConstantsValueToCSSValue = this.css.styleConstantsValueToCSSValue((StyleConstants)name, value);
            if (styleConstantsValueToCSSValue != null) {
                CSS.Attribute styleConstantsKeyToCSSKey = this.css.styleConstantsKeyToCSSKey((StyleConstants)name);
                if (styleConstantsKeyToCSSKey != null) {
                    return super.addAttribute(removeAttribute, styleConstantsKeyToCSSKey, styleConstantsValueToCSSValue);
                }
            }
        }
        return super.addAttribute(removeAttribute, name, value);
    }
    
    @Override
    public AttributeSet addAttributes(AttributeSet removeHTMLTags, AttributeSet set) {
        if (!(set instanceof HTMLDocument.TaggedAttributeSet)) {
            removeHTMLTags = this.removeHTMLTags(removeHTMLTags, set);
        }
        return super.addAttributes(removeHTMLTags, this.convertAttributeSet(set));
    }
    
    @Override
    public AttributeSet removeAttribute(AttributeSet removeAttribute, Object name) {
        if ((name instanceof StyleConstants)) {
            HTML.Tag tagForStyleConstantsKey = HTML.getTagForStyleConstantsKey((StyleConstants)name);
            if (tagForStyleConstantsKey != null) {
                removeAttribute = super.removeAttribute(removeAttribute, tagForStyleConstantsKey);
            }
            CSS.Attribute styleConstantsKeyToCSSKey = this.css.styleConstantsKeyToCSSKey((StyleConstants)name);
            if (styleConstantsKeyToCSSKey != null) {
                return super.removeAttribute(removeAttribute, styleConstantsKeyToCSSKey);
            }
        }
        return super.removeAttribute(removeAttribute, name);
    }
    
    @Override
    public AttributeSet removeAttributes(AttributeSet old, Enumeration<?> names) {
        return super.removeAttributes(old, names);
    }
    
    @Override
    public AttributeSet removeAttributes(AttributeSet removeHTMLTags, AttributeSet set) {
        if (removeHTMLTags != set) {
            removeHTMLTags = this.removeHTMLTags(removeHTMLTags, set);
        }
        return super.removeAttributes(removeHTMLTags, this.convertAttributeSet(set));
    }
    
    @Override
    protected StyleContext.SmallAttributeSet createSmallAttributeSet(AttributeSet set) {
        return new StyleSheet.SmallConversionSet(set);
    }
    
    @Override
    protected MutableAttributeSet createLargeAttributeSet(AttributeSet set) {
        return new StyleSheet.LargeConversionSet(set);
    }
    
    private AttributeSet removeHTMLTags(AttributeSet removeAttribute, AttributeSet set) {
        if (!(set instanceof StyleSheet.LargeConversionSet) && !(set instanceof StyleSheet.SmallConversionSet)) {
            Enumeration<Object> attributeNames = set.getAttributeNames();
            while (attributeNames.hasMoreElements()) {
                Object nextElement = attributeNames.nextElement();
                if ((nextElement instanceof StyleConstants)) {
                    HTML.Tag tagForStyleConstantsKey = HTML.getTagForStyleConstantsKey((StyleConstants)nextElement);
                    if (tagForStyleConstantsKey != null && removeAttribute.isDefined(tagForStyleConstantsKey)) {
                        removeAttribute = super.removeAttribute(removeAttribute, tagForStyleConstantsKey);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            return removeAttribute;
        }
        return removeAttribute;
    }
    
    AttributeSet convertAttributeSet(AttributeSet set) {
        if (set instanceof StyleSheet.LargeConversionSet || set instanceof StyleSheet.SmallConversionSet) {
            return set;
        }
        Enumeration<Object> attributeNames = set.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            if ((attributeNames.nextElement() instanceof StyleConstants)) {
                StyleSheet.LargeConversionSet set2 = new StyleSheet.LargeConversionSet();
                Enumeration<Object> attributeNames2 = set.getAttributeNames();
                while (attributeNames2.hasMoreElements()) {
                    Object nextElement = attributeNames2.nextElement();
                    Object styleConstantsValueToCSSValue = null;
                    if ((nextElement instanceof StyleConstants)) {
                        CSS.Attribute styleConstantsKeyToCSSKey = this.css.styleConstantsKeyToCSSKey((StyleConstants)nextElement);
                        if (styleConstantsKeyToCSSKey != null) {
                            styleConstantsValueToCSSValue = this.css.styleConstantsValueToCSSValue((StyleConstants)nextElement, set.getAttribute(nextElement));
                            if (styleConstantsValueToCSSValue != null) {
                                set2.addAttribute(styleConstantsKeyToCSSKey, styleConstantsValueToCSSValue);
                            }
                        }
                    }
                    if (styleConstantsValueToCSSValue == null) {
                        set2.addAttribute(nextElement, set.getAttribute(nextElement));
                    }
                    else {
                        continue;
                    }
                }
                return set2;
            }
            continue;
        }
        return set;
    }
    
    @Override
    public Font getFont(AttributeSet set) {
        return this.css.getFont(this, set, 12, this);
    }
    
    @Override
    public Color getForeground(AttributeSet set) {
        Color color = this.css.getColor(set, CSS.Attribute.COLOR);
        if (color == null) {
            return Color.black;
        }
        return color;
    }
    
    @Override
    public Color getBackground(AttributeSet set) {
        return this.css.getColor(set, CSS.Attribute.BACKGROUND_COLOR);
    }
    
    public StyleSheet.BoxPainter getBoxPainter(AttributeSet set) {
        return new StyleSheet.BoxPainter(set, this.css, this);
    }
    
    public StyleSheet.ListPainter getListPainter(AttributeSet set) {
        return new StyleSheet.ListPainter(set, this);
    }
    
    public void setBaseFontSize(int baseFontSize) {
        this.css.setBaseFontSize(baseFontSize);
    }
    
    public void setBaseFontSize(String baseFontSize) {
        this.css.setBaseFontSize(baseFontSize);
    }
    
    public static int getIndexOfSize(float n) {
        return CSS.getIndexOfSize(n, StyleSheet.sizeMapDefault);
    }
    
    public float getPointSize(int n) {
        return this.css.getPointSize(n, this);
    }
    
    public float getPointSize(String s) {
        return this.css.getPointSize(s, this);
    }
    
    public Color stringToColor(String s) {
        return CSS.stringToColor(s);
    }
    
    ImageIcon getBackgroundImage(AttributeSet set) {
        Object attribute = set.getAttribute(CSS.Attribute.BACKGROUND_IMAGE);
        if (attribute != null) {
            return ((CSS.BackgroundImage)attribute).getImage(this.getBase());
        }
        return null;
    }
    
    void addRule(String[] array, AttributeSet set, boolean b) {
        int length = array.length;
        StringBuilder sb = new StringBuilder();
        sb.append(array[0]);
        for (int i = 1; i < length; ++i) {
            sb.append(' ');
            sb.append(array[i]);
        }
        String string = sb.toString();
        Style style = this.getStyle(string);
        if (style == null) {
            Style addStyle = this.addStyle(string, null);
            synchronized (this) {
                StyleSheet.SelectorMapping selectorMapping = this.getRootSelectorMapping();
                for (int j = length - 1; j >= 0; --j) {
                    selectorMapping = selectorMapping.getChildSelectorMapping(array[j], true);
                }
                style = selectorMapping.getStyle();
                if (style == null) {
                    style = addStyle;
                    selectorMapping.setStyle(style);
                    this.refreshResolvedRules(string, array, style, selectorMapping.getSpecificity());
                }
            }
        }
        if (b) {
            style = this.getLinkedStyle(style);
        }
        style.addAttributes(set);
    }
    
    private synchronized void linkStyleSheetAt(StyleSheet styleSheet, int n) {
        if (this.resolvedStyles.size() > 0) {
            Enumeration<StyleSheet.ResolvedStyle> elements = this.resolvedStyles.elements();
            while (elements.hasMoreElements()) {
                StyleSheet.ResolvedStyle resolvedStyle = (StyleSheet.ResolvedStyle)elements.nextElement();
                resolvedStyle.insertExtendedStyleAt(styleSheet.getRule(resolvedStyle.getName()), n);
            }
        }
    }
    
    private synchronized void unlinkStyleSheet(StyleSheet styleSheet, int n) {
        if (this.resolvedStyles.size() > 0) {
            Enumeration<StyleSheet.ResolvedStyle> elements = this.resolvedStyles.elements();
            while (elements.hasMoreElements()) {
                ((StyleSheet.ResolvedStyle)elements.nextElement()).removeExtendedStyleAt(n);
            }
        }
    }
    
    String[] getSimpleSelectors(String cleanSelectorString) {
        cleanSelectorString = this.cleanSelectorString(cleanSelectorString);
        StyleSheet.SearchBuffer obtainSearchBuffer = StyleSheet.SearchBuffer.obtainSearchBuffer();
        Vector<String> vector = obtainSearchBuffer.getVector();
        int i = 0;
        int length = cleanSelectorString.length();
        while (i != -1) {
            int index = cleanSelectorString.indexOf(32, i);
            if (index != -1) {
                vector.addElement(cleanSelectorString.substring(i, index));
                if (++index == length) {
                    i = -1;
                }
                else {
                    i = index;
                }
            }
            else {
                vector.addElement(cleanSelectorString.substring(i));
                i = -1;
            }
        }
        String[] anArray = new String[vector.size()];
        vector.copyInto(anArray);
        StyleSheet.SearchBuffer.releaseSearchBuffer(obtainSearchBuffer);
        return anArray;
    }
    
    String cleanSelectorString(String s) {
        int n = 1;
        for (int i = 0; i < s.length(); ++i) {
            switch (s.charAt(i)) {
                case ' ': {
                    if (n != 0) {
                        return this._cleanSelectorString(s);
                    }
                    n = 1;
                    break;
                }
                case '\t':
                case '\n':
                case '\r': {
                    return this._cleanSelectorString(s);
                }
                default: {
                    n = 0;
                    break;
                }
            }
        }
        if (n != 0) {
            return this._cleanSelectorString(s);
        }
        return s;
    }
    
    private String _cleanSelectorString(String s) {
        StyleSheet.SearchBuffer obtainSearchBuffer = StyleSheet.SearchBuffer.obtainSearchBuffer();
        StringBuffer stringBuffer = obtainSearchBuffer.getStringBuffer();
        int n = 1;
        int offset = 0;
        char[] charArray = s.toCharArray();
        int length = charArray.length;
        String string = null;
        try {
            for (int i = 0; i < length; ++i) {
                switch (charArray[i]) {
                    case ' ': {
                        if (n == 0) {
                            n = 1;
                            if (offset < i) {
                                stringBuffer.append(charArray, offset, 1 + i - offset);
                            }
                        }
                        offset = i + 1;
                        break;
                    }
                    case '\t':
                    case '\n':
                    case '\r': {
                        if (n == 0) {
                            n = 1;
                            if (offset < i) {
                                stringBuffer.append(charArray, offset, i - offset);
                                stringBuffer.append(' ');
                            }
                        }
                        offset = i + 1;
                        break;
                    }
                    default: {
                        n = 0;
                        break;
                    }
                }
            }
            if (n != 0 && stringBuffer.length() > 0) {
                stringBuffer.setLength(stringBuffer.length() - 1);
            }
            else if (offset < length) {
                stringBuffer.append(charArray, offset, length - offset);
            }
            string = stringBuffer.toString();
        }
        finally {
            StyleSheet.SearchBuffer.releaseSearchBuffer(obtainSearchBuffer);
        }
        return string;
    }
    
    private StyleSheet.SelectorMapping getRootSelectorMapping() {
        return this.selectorMapping;
    }
    
    static int getSpecificity(String s) {
        int n = 0;
        int n2 = 1;
        for (int i = 0; i < s.length(); ++i) {
            switch (s.charAt(i)) {
                case '.': {
                    n += 100;
                    break;
                }
                case '#': {
                    n += 10000;
                    break;
                }
                case ' ': {
                    n2 = 1;
                    break;
                }
                default: {
                    if (n2 != 0) {
                        n2 = 0;
                        ++n;
                        break;
                    }
                    break;
                }
            }
        }
        return n;
    }
    
    private Style getLinkedStyle(Style style) {
        Style addStyle = (Style)style.getResolveParent();
        if (addStyle == null) {
            addStyle = this.addStyle(null, null);
            style.setResolveParent(addStyle);
            return addStyle;
        }
        return addStyle;
    }
    
    private synchronized Style getResolvedStyle(String key, Vector vector, HTML.Tag tag) {
        Style resolvedStyle = (Style)this.resolvedStyles.get(key);
        if (resolvedStyle == null) {
            resolvedStyle = this.createResolvedStyle(key, vector, tag);
            return resolvedStyle;
        }
        return resolvedStyle;
    }
    
    private synchronized Style getResolvedStyle(String key) {
        Style resolvedStyle = (Style)this.resolvedStyles.get(key);
        if (resolvedStyle == null) {
            resolvedStyle = this.createResolvedStyle(key);
            return resolvedStyle;
        }
        return resolvedStyle;
    }
    
    private void addSortedStyle(StyleSheet.SelectorMapping selectorMapping, Vector<StyleSheet.SelectorMapping> vector) {
        int size = vector.size();
        if (size > 0) {
            int specificity = selectorMapping.getSpecificity();
            int i = 0;
            while (i < size) {
                if (specificity >= ((StyleSheet.SelectorMapping)vector.elementAt(i)).getSpecificity()) {
                    vector.insertElementAt(selectorMapping, i);
                    return;
                }
                ++i;
            }
        }
        vector.addElement(selectorMapping);
    }
    
    private synchronized void getStyles(StyleSheet.SelectorMapping selectorMapping, Vector<StyleSheet.SelectorMapping> vector, String[] array, String[] array2, String[] array3, int i, int n, Hashtable<StyleSheet.SelectorMapping, StyleSheet.SelectorMapping> hashtable) {
        if (hashtable.contains(selectorMapping)) {
            return;
        }
        hashtable.put(selectorMapping, selectorMapping);
        if (selectorMapping.getStyle() != null) {
            this.addSortedStyle(selectorMapping, vector);
        }
        String s;
        StyleSheet.SelectorMapping childSelectorMapping;
        String s2;
        StyleSheet.SelectorMapping childSelectorMapping2;
        StyleSheet.SelectorMapping childSelectorMapping3;
        String s3;
        StyleSheet.SelectorMapping childSelectorMapping4;
        StyleSheet.SelectorMapping childSelectorMapping5;
        for (; i < n; ++i) {
            s = array[i];
            if (s != null) {
                childSelectorMapping = selectorMapping.getChildSelectorMapping(s, false);
                if (childSelectorMapping != null) {
                    this.getStyles(childSelectorMapping, vector, array, array2, array3, i + 1, n, hashtable);
                }
                if (array3[i] != null) {
                    s2 = array3[i];
                    childSelectorMapping2 = selectorMapping.getChildSelectorMapping(s + "." + s2, false);
                    if (childSelectorMapping2 != null) {
                        this.getStyles(childSelectorMapping2, vector, array, array2, array3, i + 1, n, hashtable);
                    }
                    childSelectorMapping3 = selectorMapping.getChildSelectorMapping("." + s2, false);
                    if (childSelectorMapping3 != null) {
                        this.getStyles(childSelectorMapping3, vector, array, array2, array3, i + 1, n, hashtable);
                    }
                }
                if (array2[i] != null) {
                    s3 = array2[i];
                    childSelectorMapping4 = selectorMapping.getChildSelectorMapping(s + "#" + s3, false);
                    if (childSelectorMapping4 != null) {
                        this.getStyles(childSelectorMapping4, vector, array, array2, array3, i + 1, n, hashtable);
                    }
                    childSelectorMapping5 = selectorMapping.getChildSelectorMapping("#" + s3, false);
                    if (childSelectorMapping5 != null) {
                        this.getStyles(childSelectorMapping5, vector, array, array2, array3, i + 1, n, hashtable);
                    }
                }
            }
        }
        return;
    }
    
    private synchronized Style createResolvedStyle(String key, String[] array, String[] array2, String[] array3) {
        StyleSheet.SearchBuffer obtainSearchBuffer = StyleSheet.SearchBuffer.obtainSearchBuffer();
        Vector<StyleSheet.SelectorMapping> vector = obtainSearchBuffer.getVector();
        Hashtable<StyleSheet.SelectorMapping, StyleSheet.SelectorMapping> hashtable = obtainSearchBuffer.getHashtable();
        try {
            StyleSheet.SelectorMapping rootSelectorMapping = this.getRootSelectorMapping();
            int length = array.length;
            String s = array[0];
            StyleSheet.SelectorMapping childSelectorMapping = rootSelectorMapping.getChildSelectorMapping(s, false);
            if (childSelectorMapping != null) {
                this.getStyles(childSelectorMapping, vector, array, array2, array3, 1, length, hashtable);
            }
            if (array3[0] != null) {
                String s2 = array3[0];
                StyleSheet.SelectorMapping childSelectorMapping2 = rootSelectorMapping.getChildSelectorMapping(s + "." + s2, false);
                if (childSelectorMapping2 != null) {
                    this.getStyles(childSelectorMapping2, vector, array, array2, array3, 1, length, hashtable);
                }
                StyleSheet.SelectorMapping childSelectorMapping3 = rootSelectorMapping.getChildSelectorMapping("." + s2, false);
                if (childSelectorMapping3 != null) {
                    this.getStyles(childSelectorMapping3, vector, array, array2, array3, 1, length, hashtable);
                }
            }
            if (array2[0] != null) {
                String s3 = array2[0];
                StyleSheet.SelectorMapping childSelectorMapping4 = rootSelectorMapping.getChildSelectorMapping(s + "#" + s3, false);
                if (childSelectorMapping4 != null) {
                    this.getStyles(childSelectorMapping4, vector, array, array2, array3, 1, length, hashtable);
                }
                StyleSheet.SelectorMapping childSelectorMapping5 = rootSelectorMapping.getChildSelectorMapping("#" + s3, false);
                if (childSelectorMapping5 != null) {
                    this.getStyles(childSelectorMapping5, vector, array, array2, array3, 1, length, hashtable);
                }
            }
            int n = (this.linkedStyleSheets != null) ? this.linkedStyleSheets.size() : 0;
            int size = vector.size();
            AttributeSet[] array4 = new AttributeSet[size + n];
            for (int i = 0; i < size; ++i) {
                array4[i] = ((StyleSheet.SelectorMapping)vector.elementAt(i)).getStyle();
            }
            for (int j = 0; j < n; ++j) {
                Style rule = ((StyleSheet)this.linkedStyleSheets.elementAt(j)).getRule(key);
                if (rule == null) {
                    array4[j + size] = SimpleAttributeSet.EMPTY;
                }
                else {
                    array4[j + size] = rule;
                }
            }
            StyleSheet.ResolvedStyle value = new StyleSheet.ResolvedStyle(key, array4, size);
            this.resolvedStyles.put(key, value);
            return value;
        }
        finally {
            StyleSheet.SearchBuffer.releaseSearchBuffer(obtainSearchBuffer);
        }
    }
    
    private Style createResolvedStyle(String s, Vector vector, HTML.Tag tag) {
        int size = vector.size();
        String[] array = new String[size];
        String[] array2 = new String[size];
        String[] array3 = new String[size];
        for (int i = 0; i < size; ++i) {
            Element element = (Element)vector.elementAt(i);
            AttributeSet attributes = element.getAttributes();
            if (i == 0 && element.isLeaf()) {
                Object attribute = attributes.getAttribute(tag);
                if ((attribute instanceof AttributeSet)) {
                    attributes = (AttributeSet)attribute;
                }
                else {
                    attributes = null;
                }
            }
            if (attributes != null) {
                HTML.Tag tag2 = (HTML.Tag)attributes.getAttribute(StyleConstants.NameAttribute);
                if (tag2 != null) {
                    array[i] = tag2.toString();
                }
                else {
                    array[i] = null;
                }
                if (attributes.isDefined(HTML.Attribute.CLASS)) {
                    array3[i] = attributes.getAttribute(HTML.Attribute.CLASS).toString();
                }
                else {
                    array3[i] = null;
                }
                if (attributes.isDefined(HTML.Attribute.ID)) {
                    array2[i] = attributes.getAttribute(HTML.Attribute.ID).toString();
                }
                else {
                    array2[i] = null;
                }
            }
            else {
                String[] array4 = array;
                int n = i;
                int n2 = i;
                int n3 = i;
                String s2 = null;
                array3[n3] = s2;
                array4[n] = (array2[n2] = s2);
            }
        }
        array[0] = tag.toString();
        return this.createResolvedStyle(s, array, array2, array3);
    }
    
    private Style createResolvedStyle(String s) {
        StyleSheet.SearchBuffer obtainSearchBuffer = StyleSheet.SearchBuffer.obtainSearchBuffer();
        Vector<String> vector = obtainSearchBuffer.getVector();
        try {
            int index = 0;
            int index2 = 0;
            int index3;
            for (int i = 0, length = s.length(); i < length; i = index3 + 1) {
                if (index == i) {
                    index = s.indexOf(46, i);
                }
                if (index2 == i) {
                    index2 = s.indexOf(35, i);
                }
                index3 = s.indexOf(32, i);
                if (index3 == -1) {
                    index3 = length;
                }
                if (index != -1 && index2 != -1 && index < index3 && index2 < index3) {
                    if (index2 < index) {
                        if (i == index2) {
                            vector.addElement("");
                        }
                        else {
                            vector.addElement(s.substring(i, index2));
                        }
                        if (index + 1 < index3) {
                            vector.addElement(s.substring(index + 1, index3));
                        }
                        else {
                            vector.addElement(null);
                        }
                        if (index2 + 1 == index) {
                            vector.addElement(null);
                        }
                        else {
                            vector.addElement(s.substring(index2 + 1, index));
                        }
                    }
                    else if (index2 < index3) {
                        if (i == index) {
                            vector.addElement("");
                        }
                        else {
                            vector.addElement(s.substring(i, index));
                        }
                        if (index + 1 < index2) {
                            vector.addElement(s.substring(index + 1, index2));
                        }
                        else {
                            vector.addElement(null);
                        }
                        if (index2 + 1 == index3) {
                            vector.addElement(null);
                        }
                        else {
                            vector.addElement(s.substring(index2 + 1, index3));
                        }
                    }
                    index2 = (index = index3 + 1);
                }
                else if (index != -1 && index < index3) {
                    if (index == i) {
                        vector.addElement("");
                    }
                    else {
                        vector.addElement(s.substring(i, index));
                    }
                    if (index + 1 == index3) {
                        vector.addElement(null);
                    }
                    else {
                        vector.addElement(s.substring(index + 1, index3));
                    }
                    vector.addElement(null);
                    index = index3 + 1;
                }
                else if (index2 != -1 && index2 < index3) {
                    if (index2 == i) {
                        vector.addElement("");
                    }
                    else {
                        vector.addElement(s.substring(i, index2));
                    }
                    vector.addElement(null);
                    if (index2 + 1 == index3) {
                        vector.addElement(null);
                    }
                    else {
                        vector.addElement(s.substring(index2 + 1, index3));
                    }
                    index2 = index3 + 1;
                }
                else {
                    vector.addElement(s.substring(i, index3));
                    vector.addElement(null);
                    vector.addElement(null);
                }
            }
            int size = vector.size();
            int n = size / 3;
            String[] array = new String[n];
            String[] array2 = new String[n];
            String[] array3 = new String[n];
            for (int j = 0, index4 = size - 3; j < n; ++j, index4 -= 3) {
                array[j] = (String)vector.elementAt(index4);
                array3[j] = (String)vector.elementAt(index4 + 1);
                array2[j] = (String)vector.elementAt(index4 + 2);
            }
            return this.createResolvedStyle(s, array, array2, array3);
        }
        finally {
            StyleSheet.SearchBuffer.releaseSearchBuffer(obtainSearchBuffer);
        }
    }
    
    private synchronized void refreshResolvedRules(String s, String[] array, Style style, int n) {
        if (this.resolvedStyles.size() > 0) {
            Enumeration<StyleSheet.ResolvedStyle> elements = this.resolvedStyles.elements();
            while (elements.hasMoreElements()) {
                StyleSheet.ResolvedStyle resolvedStyle = (StyleSheet.ResolvedStyle)elements.nextElement();
                if (resolvedStyle.matches(s)) {
                    resolvedStyle.insertStyle(style, n);
                }
                else {
                    continue;
                }
            }
        }
    }
    
    void rebaseSizeMap(int n) {
        this.sizeMap = new int[StyleSheet.sizeMapDefault.length];
        for (int i = 0; i < StyleSheet.sizeMapDefault.length; ++i) {
            this.sizeMap[i] = Math.max(n * StyleSheet.sizeMapDefault[i] / StyleSheet.sizeMapDefault[CSS.baseFontSizeIndex], 4);
        }
    }
    
    int[] getSizeMap() {
        return this.sizeMap;
    }
    
    boolean isW3CLengthUnits() {
        return this.w3cLengthUnits;
    }
    
    static {
        noBorder = new EmptyBorder(0, 0, 0, 0);
        sizeMapDefault = new int[] { 8, 10, 12, 14, 18, 24, 36 };
    }
    
    class LargeConversionSet extends SimpleAttributeSet
    {
        public LargeConversionSet(AttributeSet source) {
            super(source);
        }
        
        public LargeConversionSet() {
        }
        
        @Override
        public boolean isDefined(Object attrName) {
            if ((attrName instanceof StyleConstants)) {
                CSS.Attribute styleConstantsKeyToCSSKey = StyleSheet.this.css.styleConstantsKeyToCSSKey((StyleConstants)attrName);
                if (styleConstantsKeyToCSSKey != null) {
                    return super.isDefined(styleConstantsKeyToCSSKey);
                }
            }
            return super.isDefined(attrName);
        }
        
        @Override
        public Object getAttribute(Object name) {
            if ((name instanceof StyleConstants)) {
                CSS.Attribute styleConstantsKeyToCSSKey = StyleSheet.this.css.styleConstantsKeyToCSSKey((StyleConstants)name);
                if (styleConstantsKeyToCSSKey != null) {
                    Object attribute = super.getAttribute(styleConstantsKeyToCSSKey);
                    if (attribute != null) {
                        return StyleSheet.this.css.cssValueToStyleConstantsValue((StyleConstants)name, attribute);
                    }
                }
            }
            return super.getAttribute(name);
        }
    }
    
    class SmallConversionSet extends StyleContext.SmallAttributeSet
    {
        public SmallConversionSet(AttributeSet attrs) {
            super(attrs);
        }
        
        @Override
        public boolean isDefined(Object key) {
            if ((key instanceof StyleConstants)) {
                CSS.Attribute styleConstantsKeyToCSSKey = StyleSheet.this.css.styleConstantsKeyToCSSKey((StyleConstants)key);
                if (styleConstantsKeyToCSSKey != null) {
                    return super.isDefined(styleConstantsKeyToCSSKey);
                }
            }
            return super.isDefined(key);
        }
        
        @Override
        public Object getAttribute(Object key) {
            if ((key instanceof StyleConstants)) {
                CSS.Attribute styleConstantsKeyToCSSKey = StyleSheet.this.css.styleConstantsKeyToCSSKey((StyleConstants)key);
                if (styleConstantsKeyToCSSKey != null) {
                    Object attribute = super.getAttribute(styleConstantsKeyToCSSKey);
                    if (attribute != null) {
                        return StyleSheet.this.css.cssValueToStyleConstantsValue((StyleConstants)key, attribute);
                    }
                }
            }
            return super.getAttribute(key);
        }
    }
    
    private static class SearchBuffer
    {
        static Stack<StyleSheet.SearchBuffer> searchBuffers;
        Vector vector;
        StringBuffer stringBuffer;
        Hashtable hashtable;
        
        private SearchBuffer() {
            this.vector = null;
            this.stringBuffer = null;
            this.hashtable = null;
        }
        
        static StyleSheet.SearchBuffer obtainSearchBuffer() {
            StyleSheet.SearchBuffer searchBuffer;
            try {
                if (!StyleSheet.SearchBuffer.searchBuffers.empty()) {
                    searchBuffer = (StyleSheet.SearchBuffer)StyleSheet.SearchBuffer.searchBuffers.pop();
                }
                else {
                    searchBuffer = new StyleSheet.SearchBuffer();
                }
            }
            catch (EmptyStackException ex) {
                searchBuffer = new StyleSheet.SearchBuffer();
            }
            return searchBuffer;
        }
        
        static void releaseSearchBuffer(StyleSheet.SearchBuffer item) {
            item.empty();
            StyleSheet.SearchBuffer.searchBuffers.push(item);
        }
        
        StringBuffer getStringBuffer() {
            if (this.stringBuffer == null) {
                this.stringBuffer = new StringBuffer();
            }
            return this.stringBuffer;
        }
        
        Vector getVector() {
            if (this.vector == null) {
                this.vector = new Vector();
            }
            return this.vector;
        }
        
        Hashtable getHashtable() {
            if (this.hashtable == null) {
                this.hashtable = new Hashtable();
            }
            return this.hashtable;
        }
        
        void empty() {
            if (this.stringBuffer != null) {
                this.stringBuffer.setLength(0);
            }
            if (this.vector != null) {
                this.vector.removeAllElements();
            }
            if (this.hashtable != null) {
                this.hashtable.clear();
            }
        }
        
        static {
            StyleSheet.SearchBuffer.searchBuffers = new Stack();
        }
    }
    
    public static class BoxPainter implements Serializable
    {
        float topMargin;
        float bottomMargin;
        float leftMargin;
        float rightMargin;
        short marginFlags;
        Border border;
        Insets binsets;
        CSS css;
        StyleSheet ss;
        Color bg;
        StyleSheet.BackgroundImagePainter bgPainter;
        
        BoxPainter(AttributeSet set, CSS css, StyleSheet ss) {
            this.ss = ss;
            this.css = css;
            this.border = this.getBorder(set);
            this.binsets = this.border.getBorderInsets(null);
            this.topMargin = this.getLength(CSS.Attribute.MARGIN_TOP, set);
            this.bottomMargin = this.getLength(CSS.Attribute.MARGIN_BOTTOM, set);
            this.leftMargin = this.getLength(CSS.Attribute.MARGIN_LEFT, set);
            this.rightMargin = this.getLength(CSS.Attribute.MARGIN_RIGHT, set);
            this.bg = ss.getBackground(set);
            if (ss.getBackgroundImage(set) != null) {
                this.bgPainter = new StyleSheet.BackgroundImagePainter(set, css, ss);
            }
        }
        
        Border getBorder(AttributeSet set) {
            return new CSSBorder(set);
        }
        
        Color getBorderColor(AttributeSet set) {
            Color color = this.css.getColor(set, CSS.Attribute.BORDER_COLOR);
            if (color == null) {
                color = this.css.getColor(set, CSS.Attribute.COLOR);
                if (color == null) {
                    return Color.black;
                }
            }
            return color;
        }
        
        public float getInset(int i, View view) {
            AttributeSet attributes = view.getAttributes();
            float n = 0.0f;
            float n2;
            switch (i) {
                case 2: {
                    n2 = n + this.getOrientationMargin(StyleSheet.BoxPainter.HorizontalMargin.LEFT, this.leftMargin, attributes, isLeftToRight(view)) + (float)this.binsets.left + this.getLength(CSS.Attribute.PADDING_LEFT, attributes);
                    break;
                }
                case 4: {
                    n2 = n + this.getOrientationMargin(StyleSheet.BoxPainter.HorizontalMargin.RIGHT, this.rightMargin, attributes, isLeftToRight(view)) + (float)this.binsets.right + this.getLength(CSS.Attribute.PADDING_RIGHT, attributes);
                    break;
                }
                case 1: {
                    n2 = n + this.topMargin + (float)this.binsets.top + this.getLength(CSS.Attribute.PADDING_TOP, attributes);
                    break;
                }
                case 3: {
                    n2 = n + this.bottomMargin + (float)this.binsets.bottom + this.getLength(CSS.Attribute.PADDING_BOTTOM, attributes);
                    break;
                }
                default: {
                    throw new IllegalArgumentException("Invalid side: " + i);
                }
            }
            return n2;
        }
        
        public void paint(Graphics graphics, float n, float n2, float n3, float n4, View view) {
            float n5 = 0.0f;
            float topMargin = 0.0f;
            float n6 = 0.0f;
            float n7 = 0.0f;
            AttributeSet attributes = view.getAttributes();
            int leftToRight = isLeftToRight(view);
            float orientationMargin = this.getOrientationMargin(StyleSheet.BoxPainter.HorizontalMargin.LEFT, this.leftMargin, attributes, (boolean)leftToRight);
            float orientationMargin2 = this.getOrientationMargin(StyleSheet.BoxPainter.HorizontalMargin.RIGHT, this.rightMargin, attributes, (boolean)leftToRight);
            if (!(view instanceof HTMLEditorKit.HTMLFactory.BodyBlockView)) {
                n5 = orientationMargin;
                topMargin = this.topMargin;
                n6 = -(orientationMargin + orientationMargin2);
                n7 = -(this.topMargin + this.bottomMargin);
            }
            if (this.bg != null) {
                graphics.setColor(this.bg);
                graphics.fillRect((int)(n + n5), (int)(n2 + topMargin), (int)(n3 + n6), (int)(n4 + n7));
            }
            if (this.bgPainter != null) {
                this.bgPainter.paint(graphics, n + n5, n2 + topMargin, n3 + n6, n4 + n7, view);
            }
            n += orientationMargin;
            n2 += this.topMargin;
            n3 -= orientationMargin + orientationMargin2;
            n4 -= this.topMargin + this.bottomMargin;
            if ((this.border instanceof BevelBorder)) {
                for (int i = (int)this.getLength(CSS.Attribute.BORDER_TOP_WIDTH, attributes) - 1; i >= 0; --i) {
                    this.border.paintBorder(null, graphics, (int)n + i, (int)n2 + i, (int)n3 - 2 * i, (int)n4 - 2 * i);
                }
            }
            else {
                this.border.paintBorder(null, graphics, (int)n, (int)n2, (int)n3, (int)n4);
            }
        }
        
        float getLength(CSS.Attribute attribute, AttributeSet set) {
            return this.css.getLength(set, attribute, this.ss);
        }
        
        static boolean isLeftToRight(View view) {
            boolean leftToRight = true;
            Container container;
            if (isOrientationAware(view) && view != null && (container = view.getContainer()) != null) {
                leftToRight = container.getComponentOrientation().isLeftToRight();
            }
            return leftToRight;
        }
        
        static boolean isOrientationAware(View view) {
            boolean n = false;
            AttributeSet attributes;
            Object attribute;
            if (view != null && (attributes = view.getElement().getAttributes()) != null && (attribute = attributes.getAttribute(StyleConstants.NameAttribute)) instanceof HTML.Tag && (attribute == HTML.Tag.DIR || attribute == HTML.Tag.MENU || attribute == HTML.Tag.UL || attribute == HTML.Tag.OL)) {
                n = true;
            }
            return n;
        }
        
        float getOrientationMargin(StyleSheet.BoxPainter.HorizontalMargin horizontalMargin, float n, AttributeSet set, boolean b) {
            n = (n = n);
            Object o = null;
            switch (horizontalMargin) {
                case RIGHT: {
                    n = (b ? this.getLength(CSS.Attribute.MARGIN_RIGHT_LTR, set) : this.getLength(CSS.Attribute.MARGIN_RIGHT_RTL, set));
                    o = set.getAttribute(CSS.Attribute.MARGIN_RIGHT);
                    break;
                }
                case LEFT: {
                    n = (b ? this.getLength(CSS.Attribute.MARGIN_LEFT_LTR, set) : this.getLength(CSS.Attribute.MARGIN_LEFT_RTL, set));
                    o = set.getAttribute(CSS.Attribute.MARGIN_LEFT);
                    break;
                }
            }
            if (o == null && n != -2.1474836E9f) {
            }
            return n;
        }
        
        enum HorizontalMargin
        {
            LEFT, 
            RIGHT;
        }
    }
    
    public static class ListPainter implements Serializable
    {
        static final char[][] romanChars;
        private Rectangle paintRect;
        private boolean checkedForStart;
        private int start;
        private CSS.Value type;
        URL imageurl;
        private StyleSheet ss;
        Icon img;
        private int bulletgap;
        private boolean isLeftToRight;
        
        ListPainter(AttributeSet set, StyleSheet ss) {
            this.ss = null;
            this.img = null;
            this.bulletgap = 5;
            this.ss = ss;
            String str = (String)set.getAttribute(CSS.Attribute.LIST_STYLE_IMAGE);
            this.type = null;
            if (str != null && !str.equals("none")) {
                String s = null;
                try {
                    StringTokenizer stringTokenizer = new StringTokenizer(str, "()");
                    if (stringTokenizer.hasMoreTokens()) {
                        s = stringTokenizer.nextToken();
                    }
                    if (stringTokenizer.hasMoreTokens()) {
                        s = stringTokenizer.nextToken();
                    }
                    this.img = new ImageIcon(new URL(s));
                }
                catch (MalformedURLException ex) {
                    if (s != null && ss != null && ss.getBase() != null) {
                        try {
                            this.img = new ImageIcon(new URL(ss.getBase(), s));
                        }
                        catch (MalformedURLException ex2) {
                            this.img = null;
                        }
                    }
                    else {
                        this.img = null;
                    }
                }
            }
            if (this.img == null) {
                this.type = (CSS.Value)set.getAttribute(CSS.Attribute.LIST_STYLE_TYPE);
            }
            this.start = 1;
            this.paintRect = new Rectangle();
        }
        
        private CSS.Value getChildType(View view) {
            CSS.Value value = (CSS.Value)view.getAttributes().getAttribute(CSS.Attribute.LIST_STYLE_TYPE);
            if (value == null) {
                if (this.type == null) {
                    View parent = view.getParent();
                    HTMLDocument htmlDocument = (HTMLDocument)parent.getDocument();
                    if (HTMLDocument.matchNameAttribute(parent.getElement().getAttributes(), HTML.Tag.OL)) {
                        value = CSS.Value.DECIMAL;
                    }
                    else {
                        value = CSS.Value.DISC;
                    }
                }
                else {
                    value = this.type;
                }
            }
            return value;
        }
        
        private void getStart(View view) {
            this.checkedForStart = true;
            Element element = view.getElement();
            if (element != null) {
                AttributeSet attributes = element.getAttributes();
                Object attribute;
                if (attributes != null && attributes.isDefined(HTML.Attribute.START) && (attribute = attributes.getAttribute(HTML.Attribute.START)) != null && attribute instanceof String) {
                    try {
                        this.start = Integer.parseInt((String)attribute);
                    }
                    catch (NumberFormatException ex) {}
                }
            }
        }
        
        private int getRenderIndex(View view, int i) {
            if (!this.checkedForStart) {
                this.getStart(view);
            }
            AttributeSet attributes;
            Object attribute;
            for (i = (i = i); i >= 0; --i) {
                attributes = view.getElement().getElement(i).getAttributes();
                if (attributes.getAttribute(StyleConstants.NameAttribute) != HTML.Tag.LI) {
                    --i;
                }
                else if (attributes.isDefined(HTML.Attribute.VALUE)) {
                    attribute = attributes.getAttribute(HTML.Attribute.VALUE);
                    if (attribute != null && attribute instanceof String) {
                        try {
                            return i - i + Integer.parseInt((String)attribute);
                        }
                        catch (NumberFormatException ex) {}
                    }
                }
            }
            return i + this.start;
        }
        
        public void paint(Graphics graphics, float n, float n2, float n3, float n4, View view, int n5) {
            View view2 = view.getView(n5);
            Container container = view.getContainer();
            Object attribute = view2.getElement().getAttributes().getAttribute(StyleConstants.NameAttribute);
            if (!(attribute instanceof HTML.Tag) || attribute != HTML.Tag.LI) {
                return;
            }
            this.isLeftToRight = container.getComponentOrientation().isLeftToRight();
            float alignment = 0.0f;
            if (view2.getViewCount() > 0) {
                View view3 = view2.getView(0);
                Object attribute2 = view3.getElement().getAttributes().getAttribute(StyleConstants.NameAttribute);
                if ((attribute2 == HTML.Tag.P || attribute2 == HTML.Tag.IMPLIED) && view3.getViewCount() > 0) {
                    this.paintRect.setBounds((int)n, (int)n2, (int)n3, (int)n4);
                    Shape childAllocation = view2.getChildAllocation(0, this.paintRect);
                    Shape childAllocation2;
                    if (childAllocation != null && (childAllocation2 = view3.getView(0).getChildAllocation(0, childAllocation)) != null) {
                        Rectangle rectangle = (childAllocation2 instanceof Rectangle) ? ((Rectangle)childAllocation2) : childAllocation2.getBounds();
                        alignment = view3.getView(0).getAlignment(1);
                        n2 = (float)rectangle.y;
                        n4 = (float)rectangle.height;
                    }
                }
            }
            graphics.setColor(container.isEnabled() ? ((this.ss != null) ? this.ss.getForeground(view2.getAttributes()) : container.getForeground()) : UIManager.getColor("textInactiveText"));
            if (this.img != null) {
                this.drawIcon(graphics, (int)n, (int)n2, (int)n3, (int)n4, alignment, container);
                return;
            }
            CSS.Value childType = this.getChildType(view2);
            Font font = ((StyledDocument)view2.getDocument()).getFont(view2.getAttributes());
            if (font != null) {
                graphics.setFont(font);
            }
            if (childType == CSS.Value.SQUARE || childType == CSS.Value.CIRCLE || childType == CSS.Value.DISC) {
                this.drawShape(graphics, childType, (int)n, (int)n2, (int)n3, (int)n4, alignment);
            }
            else if (childType == CSS.Value.DECIMAL) {
                this.drawLetter(graphics, '1', (int)n, (int)n2, (int)n3, (int)n4, alignment, this.getRenderIndex(view, n5));
            }
            else if (childType == CSS.Value.LOWER_ALPHA) {
                this.drawLetter(graphics, 'a', (int)n, (int)n2, (int)n3, (int)n4, alignment, this.getRenderIndex(view, n5));
            }
            else if (childType == CSS.Value.UPPER_ALPHA) {
                this.drawLetter(graphics, 'A', (int)n, (int)n2, (int)n3, (int)n4, alignment, this.getRenderIndex(view, n5));
            }
            else if (childType == CSS.Value.LOWER_ROMAN) {
                this.drawLetter(graphics, 'i', (int)n, (int)n2, (int)n3, (int)n4, alignment, this.getRenderIndex(view, n5));
            }
            else if (childType == CSS.Value.UPPER_ROMAN) {
                this.drawLetter(graphics, 'I', (int)n, (int)n2, (int)n3, (int)n4, alignment, this.getRenderIndex(view, n5));
            }
            return;
        }
        
        void drawIcon(Graphics graphics, int n, int a, int n2, int n3, float n4, Component component) {
            this.img.paintIcon(component, graphics, n + (this.isLeftToRight ? (-(this.img.getIconWidth() + this.bulletgap)) : (n2 + this.bulletgap)), Math.max(a, a + (int)(n4 * (float)n3) - this.img.getIconHeight()));
        }
        
        void drawShape(Graphics graphics, CSS.Value value, int n, int a, int n2, int n3, float n4) {
            int x = n + (this.isLeftToRight ? (-(this.bulletgap + 8)) : (n2 + this.bulletgap));
            int max = Math.max(a, a + (int)(n4 * (float)n3) - 8);
            if (value == CSS.Value.SQUARE) {
                graphics.drawRect(x, max, 8, 8);
            }
            else if (value == CSS.Value.CIRCLE) {
                graphics.drawOval(x, max, 8, 8);
            }
            else {
                graphics.fillOval(x, max, 8, 8);
            }
        }
        
        void drawLetter(Graphics graphics, char c, int n, int n2, int n3, int n4, float n5, int n6) {
            String formatItemNum = this.formatItemNum(n6, c);
            String s = this.isLeftToRight ? (formatItemNum + ".") : ("." + formatItemNum);
            FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(null, graphics);
            int stringWidth = SwingUtilities2.stringWidth(null, fontMetrics, s);
            SwingUtilities2.drawString(null, graphics, s, n + (this.isLeftToRight ? (-(stringWidth + this.bulletgap)) : (n3 + this.bulletgap)), Math.max(n2 + fontMetrics.getAscent(), n2 + (int)((float)n4 * n5)));
        }
        
        String formatItemNum(int i, char c) {
            boolean n = false;
            String s;
            switch (c) {
                case 'A': {
                    n = true;
                }
                case 'a': {
                    s = this.formatAlphaNumerals(i);
                    break;
                }
                case 'I': {
                    n = true;
                }
                case 'i': {
                    s = this.formatRomanNumerals(i);
                    break;
                }
                default: {
                    s = String.valueOf(i);
                    break;
                }
            }
            if (!(n == 0)) {
                s = s.toUpperCase();
                return s;
            }
            return s;
        }
        
        String formatAlphaNumerals(int n) {
            if (n > 26) {
                return this.formatAlphaNumerals(n / 26) + this.formatAlphaNumerals(n % 26);
            }
            return String.valueOf((char)(97 + n - 1));
        }
        
        String formatRomanNumerals(int n) {
            return this.formatRomanNumerals(0, n);
        }
        
        String formatRomanNumerals(int n, int n2) {
            if (n2 < 10) {
                return this.formatRomanDigit(n, n2);
            }
            return this.formatRomanNumerals(n + 1, n2 / 10) + this.formatRomanDigit(n, n2 % 10);
        }
        
        String formatRomanDigit(int n, int n2) {
            String s = "";
            if (n2 == 9) {
                return s + StyleSheet.ListPainter.romanChars[n][0] + StyleSheet.ListPainter.romanChars[n + 1][0];
            }
            if (n2 == 4) {
                return s + StyleSheet.ListPainter.romanChars[n][0] + StyleSheet.ListPainter.romanChars[n][1];
            }
            if (n2 >= 5) {
                s += StyleSheet.ListPainter.romanChars[n][1];
                n2 -= 5;
            }
            for (int i = 0; i < n2; ++i) {
                s += StyleSheet.ListPainter.romanChars[n][0];
            }
            return s;
        }
        
        static {
            romanChars = new char[][] { { 'i', 'v' }, { 'x', 'l' }, { 'c', 'd' }, { 'm', '?' } };
        }
    }
    
    static class BackgroundImagePainter implements Serializable
    {
        ImageIcon backgroundImage;
        float hPosition;
        float vPosition;
        short flags;
        private int paintX;
        private int paintY;
        private int paintMaxX;
        private int paintMaxY;
        
        BackgroundImagePainter(AttributeSet set, CSS css, StyleSheet styleSheet) {
            this.backgroundImage = styleSheet.getBackgroundImage(set);
            CSS.BackgroundPosition backgroundPosition = (CSS.BackgroundPosition)set.getAttribute(CSS.Attribute.BACKGROUND_POSITION);
            if (backgroundPosition != null) {
                this.hPosition = backgroundPosition.getHorizontalPosition();
                this.vPosition = backgroundPosition.getVerticalPosition();
                if (backgroundPosition.isHorizontalPositionRelativeToSize()) {
                    this.flags |= 0x4;
                }
                else if (backgroundPosition.isHorizontalPositionRelativeToSize()) {
                    this.hPosition *= (float)CSS.getFontSize(set, 12, styleSheet);
                }
                if (backgroundPosition.isVerticalPositionRelativeToSize()) {
                    this.flags |= 0x8;
                }
                else if (backgroundPosition.isVerticalPositionRelativeToFontSize()) {
                    this.vPosition *= (float)CSS.getFontSize(set, 12, styleSheet);
                }
            }
            CSS.Value value = (CSS.Value)set.getAttribute(CSS.Attribute.BACKGROUND_REPEAT);
            if (value == null || value == CSS.Value.BACKGROUND_REPEAT) {
                this.flags |= 0x3;
            }
            else if (value == CSS.Value.BACKGROUND_REPEAT_X) {
                this.flags |= 0x1;
            }
            else if (value == CSS.Value.BACKGROUND_REPEAT_Y) {
                this.flags |= 0x2;
            }
        }
        
        void paint(Graphics graphics, float n, float n2, float n3, float n4, View view) {
            Rectangle clipRect = graphics.getClipRect();
            if (clipRect != null) {
                graphics.clipRect((int)n, (int)n2, (int)n3, (int)n4);
            }
            if ((this.flags & 0x3) == 0x0) {
                int iconWidth2 = this.backgroundImage.getIconWidth();
                int iconWidth3 = this.backgroundImage.getIconWidth();
                if ((this.flags & 0x4) == 0x4) {
                    this.paintX = (int)(n + n3 * this.hPosition - (float)iconWidth2 * this.hPosition);
                }
                else {
                    this.paintX = (int)n + (int)this.hPosition;
                }
                if ((this.flags & 0x8) == 0x8) {
                    this.paintY = (int)(n2 + n4 * this.vPosition - (float)iconWidth3 * this.vPosition);
                }
                else {
                    this.paintY = (int)n2 + (int)this.vPosition;
                }
                if (clipRect == null || (this.paintX + iconWidth2 > clipRect.x && this.paintY + iconWidth3 > clipRect.y && this.paintX < clipRect.x + clipRect.width && this.paintY < clipRect.y + clipRect.height)) {
                    this.backgroundImage.paintIcon(null, graphics, this.paintX, this.paintY);
                }
            }
            else {
                int iconWidth = this.backgroundImage.getIconWidth();
                int iconHeight = this.backgroundImage.getIconHeight();
                if (iconWidth > 0 && iconHeight > 0) {
                    this.paintX = (int)n;
                    this.paintY = (int)n2;
                    this.paintMaxX = (int)(n + n3);
                    this.paintMaxY = (int)(n2 + n4);
                    if (this.updatePaintCoordinates(clipRect, iconWidth, iconHeight)) {
                        while (this.paintX < this.paintMaxX) {
                            for (int i = this.paintY; i < this.paintMaxY; i += iconHeight) {
                                this.backgroundImage.paintIcon(null, graphics, this.paintX, i);
                            }
                            this.paintX += iconWidth;
                        }
                    }
                }
            }
            if (clipRect != null) {
                graphics.setClip(clipRect.x, clipRect.y, clipRect.width, clipRect.height);
            }
        }
        
        private boolean updatePaintCoordinates(Rectangle rectangle, int n, int n2) {
            if ((this.flags & 0x3) == 0x1) {
                this.paintMaxY = this.paintY + 1;
            }
            else if ((this.flags & 0x3) == 0x2) {
                this.paintMaxX = this.paintX + 1;
            }
            if (rectangle != null) {
                if ((this.flags & 0x3) == 0x1 && (this.paintY + n2 <= rectangle.y || this.paintY > rectangle.y + rectangle.height)) {
                    return false;
                }
                if ((this.flags & 0x3) == 0x2 && (this.paintX + n <= rectangle.x || this.paintX > rectangle.x + rectangle.width)) {
                    return false;
                }
                if ((this.flags & 0x1) == 0x1) {
                    if (rectangle.x + rectangle.width < this.paintMaxX) {
                        if ((rectangle.x + rectangle.width - this.paintX) % n == 0) {
                            this.paintMaxX = rectangle.x + rectangle.width;
                        }
                        else {
                            this.paintMaxX = ((rectangle.x + rectangle.width - this.paintX) / n + 1) * n + this.paintX;
                        }
                    }
                    if (rectangle.x > this.paintX) {
                        this.paintX += (rectangle.x - this.paintX) / n * n;
                    }
                }
                if ((this.flags & 0x2) == 0x2) {
                    if (rectangle.y + rectangle.height < this.paintMaxY) {
                        if ((rectangle.y + rectangle.height - this.paintY) % n2 == 0) {
                            this.paintMaxY = rectangle.y + rectangle.height;
                        }
                        else {
                            this.paintMaxY = ((rectangle.y + rectangle.height - this.paintY) / n2 + 1) * n2 + this.paintY;
                        }
                    }
                    if (rectangle.y > this.paintY) {
                        this.paintY += (rectangle.y - this.paintY) / n2 * n2;
                    }
                }
            }
            return true;
        }
    }
    
    class ViewAttributeSet extends MuxingAttributeSet
    {
        View host;
        
        ViewAttributeSet(View host) {
            this.host = host;
            Document document = host.getDocument();
            StyleSheet.SearchBuffer obtainSearchBuffer = StyleSheet.SearchBuffer.obtainSearchBuffer();
            Vector vector = obtainSearchBuffer.getVector();
            try {
                if ((document instanceof HTMLDocument)) {
                    Element element = host.getElement();
                    AttributeSet attributes = element.getAttributes();
                    AttributeSet translateHTMLToCSS = StyleSheet.this.translateHTMLToCSS(attributes);
                    if (translateHTMLToCSS.getAttributeCount() != 0) {
                        vector.addElement(translateHTMLToCSS);
                    }
                    if (element.isLeaf()) {
                        Enumeration<Object> attributeNames = attributes.getAttributeNames();
                        while (attributeNames.hasMoreElements()) {
                            Object nextElement = attributeNames.nextElement();
                            if ((nextElement instanceof HTML.Tag)) {
                                if (nextElement == HTML.Tag.A) {
                                    Object attribute = attributes.getAttribute(nextElement);
                                    if (attribute != null && attribute instanceof AttributeSet && ((AttributeSet)attribute).getAttribute(HTML.Attribute.HREF) == null) {
                                        continue;
                                    }
                                }
                                Style rule2 = StyleSheet.this.getRule((HTML.Tag)nextElement, element);
                                if (rule2 != null) {
                                    vector.addElement(rule2);
                                }
                                else {
                                    continue;
                                }
                            }
                            else {
                                continue;
                            }
                        }
                    }
                    else {
                        Style rule = StyleSheet.this.getRule((HTML.Tag)attributes.getAttribute(StyleConstants.NameAttribute), element);
                        if (rule != null) {
                            vector.addElement(rule);
                        }
                    }
                }
                AttributeSet[] array = new AttributeSet[vector.size()];
                vector.copyInto(array);
                this.setAttributes(array);
            }
            finally {
                StyleSheet.SearchBuffer.releaseSearchBuffer(obtainSearchBuffer);
            }
        }
        
        @Override
        public boolean isDefined(Object o) {
            if ((o instanceof StyleConstants)) {
                CSS.Attribute styleConstantsKeyToCSSKey = StyleSheet.this.css.styleConstantsKeyToCSSKey((StyleConstants)o);
                if (styleConstantsKeyToCSSKey != null) {
                    o = styleConstantsKeyToCSSKey;
                }
            }
            return super.isDefined(o);
        }
        
        @Override
        public Object getAttribute(Object o) {
            if ((o instanceof StyleConstants)) {
                CSS.Attribute styleConstantsKeyToCSSKey = StyleSheet.this.css.styleConstantsKeyToCSSKey((StyleConstants)o);
                if (styleConstantsKeyToCSSKey != null) {
                    Object doGetAttribute = this.doGetAttribute(styleConstantsKeyToCSSKey);
                    if ((doGetAttribute instanceof CSS.CssValue)) {
                        return ((CSS.CssValue)doGetAttribute).toStyleConstants((StyleConstants)o, this.host);
                    }
                }
            }
            return this.doGetAttribute(o);
        }
        
        Object doGetAttribute(Object o) {
            Object attribute = super.getAttribute(o);
            if (attribute != null) {
                return attribute;
            }
            if (o instanceof CSS.Attribute && ((CSS.Attribute)o).isInherited()) {
                AttributeSet resolveParent = this.getResolveParent();
                if (resolveParent != null) {
                    return resolveParent.getAttribute(o);
                }
            }
            return null;
        }
        
        @Override
        public AttributeSet getResolveParent() {
            if (this.host == null) {
                return null;
            }
            View parent = this.host.getParent();
            return (parent != null) ? parent.getAttributes() : null;
        }
    }
    
    static class ResolvedStyle extends MuxingAttributeSet implements Serializable, Style
    {
        String name;
        private int extendedIndex;
        
        ResolvedStyle(String name, AttributeSet[] array, int extendedIndex) {
            super(array);
            this.name = name;
            this.extendedIndex = extendedIndex;
        }
        
        synchronized void insertStyle(Style style, int n) {
            AttributeSet[] attributes = this.getAttributes();
            int length = attributes.length;
            int n2;
            for (n2 = 0; n2 < this.extendedIndex && n <= StyleSheet.getSpecificity(((Style)attributes[n2]).getName()); ++n2) {}
            this.insertAttributeSetAt(style, n2);
            ++this.extendedIndex;
        }
        
        synchronized void removeStyle(Style style) {
            AttributeSet[] attributes = this.getAttributes();
            int i = attributes.length - 1;
            while (i >= 0) {
                if (attributes[i] == style) {
                    this.removeAttributeSetAt(i);
                    if (i < this.extendedIndex) {
                        --this.extendedIndex;
                        break;
                    }
                    break;
                }
                else {
                    --i;
                }
            }
        }
        
        synchronized void insertExtendedStyleAt(Style style, int n) {
            this.insertAttributeSetAt(style, this.extendedIndex + n);
        }
        
        synchronized void addExtendedStyle(Style style) {
            this.insertAttributeSetAt(style, this.getAttributes().length);
        }
        
        synchronized void removeExtendedStyleAt(int n) {
            this.removeAttributeSetAt(this.extendedIndex + n);
        }
        
        protected boolean matches(String s) {
            int length = s.length();
            if (length == 0) {
                return false;
            }
            int length2 = this.name.length();
            int i = s.lastIndexOf(32);
            int n = this.name.lastIndexOf(32);
            if (i >= 0) {
                ++i;
            }
            if (n >= 0) {
                ++n;
            }
            if (!this.matches(s, i, length, n, length2)) {
                return false;
            }
            while (i != -1) {
                int n2 = i - 1;
                i = s.lastIndexOf(32, n2 - 1);
                if (i >= 0) {
                    ++i;
                }
                int matches;
                int n3;
                for (matches = false; matches == 0 && n != -1; matches = this.matches(s, i, n2, n, n3)) {
                    n3 = n - 1;
                    n = this.name.lastIndexOf(32, n3 - 1);
                    if (n >= 0) {
                        ++n;
                    }
                }
                if (matches == 0) {
                    return false;
                }
                continue;
            }
            return true;
        }
        
        boolean matches(String s, int max, int n, int max2, int n2) {
            max = Math.max(max, 0);
            max2 = Math.max(max2, 0);
            int boundedIndex = this.boundedIndexOf(this.name, '.', max2, n2);
            int boundedIndex2 = this.boundedIndexOf(this.name, '#', max2, n2);
            int boundedIndex3 = this.boundedIndexOf(s, '.', max, n);
            int boundedIndex4 = this.boundedIndexOf(s, '#', max, n);
            if (boundedIndex3 != -1) {
                if (boundedIndex == -1) {
                    return false;
                }
                if (max == boundedIndex3) {
                    if (n2 - boundedIndex != n - boundedIndex3 || !s.regionMatches(max, this.name, boundedIndex, n2 - boundedIndex)) {
                        return false;
                    }
                }
                else if (n - max != n2 - max2 || !s.regionMatches(max, this.name, max2, n2 - max2)) {
                    return false;
                }
                return true;
            }
            else if (boundedIndex4 != -1) {
                if (boundedIndex2 == -1) {
                    return false;
                }
                if (max == boundedIndex4) {
                    if (n2 - boundedIndex2 != n - boundedIndex4 || !s.regionMatches(max, this.name, boundedIndex2, n2 - boundedIndex2)) {
                        return false;
                    }
                }
                else if (n - max != n2 - max2 || !s.regionMatches(max, this.name, max2, n2 - max2)) {
                    return false;
                }
                return true;
            }
            else {
                if (boundedIndex != -1) {
                    return boundedIndex - max2 == n - max && s.regionMatches(max, this.name, max2, boundedIndex - max2);
                }
                if (boundedIndex2 != -1) {
                    return boundedIndex2 - max2 == n - max && s.regionMatches(max, this.name, max2, boundedIndex2 - max2);
                }
                return n2 - max2 == n - max && s.regionMatches(max, this.name, max2, n2 - max2);
            }
        }
        
        int boundedIndexOf(String s, char ch, int fromIndex, int n) {
            int index = s.indexOf((int)ch, fromIndex);
            if (index >= n) {
                return -1;
            }
            return index;
        }
        
        @Override
        public void addAttribute(Object o, Object o2) {
        }
        
        @Override
        public void addAttributes(AttributeSet set) {
        }
        
        @Override
        public void removeAttribute(Object o) {
        }
        
        @Override
        public void removeAttributes(Enumeration<?> enumeration) {
        }
        
        @Override
        public void removeAttributes(AttributeSet set) {
        }
        
        @Override
        public void setResolveParent(AttributeSet set) {
        }
        
        @Override
        public String getName() {
            return this.name;
        }
        
        @Override
        public void addChangeListener(ChangeListener changeListener) {
        }
        
        @Override
        public void removeChangeListener(ChangeListener changeListener) {
        }
        
        public ChangeListener[] getChangeListeners() {
            return new ChangeListener[0];
        }
    }
    
    static class SelectorMapping implements Serializable
    {
        private int specificity;
        private Style style;
        private HashMap<String, StyleSheet.SelectorMapping> children;
        
        public SelectorMapping(int specificity) {
            this.specificity = specificity;
        }
        
        public int getSpecificity() {
            return this.specificity;
        }
        
        public void setStyle(Style style) {
            this.style = style;
        }
        
        public Style getStyle() {
            return this.style;
        }
        
        public StyleSheet.SelectorMapping getChildSelectorMapping(String s, boolean b) {
            StyleSheet.SelectorMapping childSelectorMapping = null;
            if (this.children != null) {
                childSelectorMapping = (StyleSheet.SelectorMapping)this.children.get(s);
            }
            else if (b) {
                this.children = new HashMap(7);
            }
            if (childSelectorMapping == null && b) {
                childSelectorMapping = this.createChildSelectorMapping(this.getChildSpecificity(s));
                this.children.put(s, childSelectorMapping);
                return childSelectorMapping;
            }
            return childSelectorMapping;
        }
        
        protected StyleSheet.SelectorMapping createChildSelectorMapping(int n) {
            return new SelectorMapping(n);
        }
        
        protected int getChildSpecificity(String s) {
            int char1 = s.charAt(0);
            int specificity = this.getSpecificity();
            if (char1 == '.') {
                specificity += 100;
            }
            else if (char1 == '#') {
                specificity += 10000;
            }
            else {
                ++specificity;
                if (s.indexOf(46) != -1) {
                    specificity += 100;
                }
                if (s.indexOf(35) != -1) {
                    specificity += 10000;
                }
            }
            return specificity;
        }
    }
    
    class CssParser implements CSSParser.CSSParserCallback
    {
        Vector<String[]> selectors;
        Vector<String> selectorTokens;
        String propertyName;
        MutableAttributeSet declaration;
        boolean parsingDeclaration;
        boolean isLink;
        URL base;
        CSSParser parser;
        
        CssParser() {
            this.selectors = new Vector();
            this.selectorTokens = new Vector();
            this.declaration = new SimpleAttributeSet();
            this.parser = new CSSParser();
        }
        
        public AttributeSet parseDeclaration(String s) {
            try {
                return this.parseDeclaration(new StringReader(s));
            }
            catch (IOException ex) {
                return null;
            }
        }
        
        public AttributeSet parseDeclaration(Reader reader) throws IOException {
            this.parse(this.base, reader, true, false);
            return this.declaration.copyAttributes();
        }
        
        public void parse(URL base, Reader reader, boolean parsingDeclaration, boolean isLink) throws IOException {
            this.base = base;
            this.isLink = isLink;
            this.parsingDeclaration = parsingDeclaration;
            this.declaration.removeAttributes(this.declaration);
            this.selectorTokens.removeAllElements();
            this.selectors.removeAllElements();
            this.propertyName = null;
            this.parser.parse(reader, this, parsingDeclaration);
        }
        
        @Override
        public void handleImport(String s) {
            URL url = CSS.getURL(this.base, s);
            if (url != null) {
                StyleSheet.this.importStyleSheet(url);
            }
        }
        
        @Override
        public void handleSelector(String s) {
            if (!s.startsWith(".") && !s.startsWith("#")) {
                s = s.toLowerCase();
            }
            int length = s.length();
            if (s.endsWith(",")) {
                if (length > 1) {
                    s = s.substring(0, length - 1);
                    this.selectorTokens.addElement(s);
                }
                this.addSelector();
            }
            else if (length > 0) {
                this.selectorTokens.addElement(s);
            }
        }
        
        @Override
        public void startRule() {
            if (this.selectorTokens.size() > 0) {
                this.addSelector();
            }
            this.propertyName = null;
        }
        
        @Override
        public void handleProperty(String propertyName) {
            this.propertyName = propertyName;
        }
        
        @Override
        public void handleValue(String string) {
            if (this.propertyName != null && string != null && string.length() > 0) {
                CSS.Attribute attribute = CSS.getAttribute(this.propertyName);
                if (attribute != null) {
                    if (attribute == CSS.Attribute.LIST_STYLE_IMAGE && string != null && !string.equals("none")) {
                        URL url = CSS.getURL(this.base, string);
                        if (url != null) {
                            string = url.toString();
                        }
                    }
                    StyleSheet.this.addCSSAttribute(this.declaration, attribute, string);
                }
                this.propertyName = null;
            }
        }
        
        @Override
        public void endRule() {
            for (int size = this.selectors.size(), i = 0; i < size; ++i) {
                String[] array = (String[])this.selectors.elementAt(i);
                if (array.length > 0) {
                    StyleSheet.this.addRule(array, this.declaration, this.isLink);
                }
            }
            this.declaration.removeAttributes(this.declaration);
            this.selectors.removeAllElements();
        }
        
        private void addSelector() {
            String[] array = new String[this.selectorTokens.size()];
            this.selectorTokens.copyInto(array);
            this.selectors.addElement(array);
            this.selectorTokens.removeAllElements();
        }
    }
}
